@@ -22,12 +22,16 @@ from extrude.util import SubDagSpec, split_dag
 PARAM_TO_MALL_MAP_ATTR = 'param_to_mall_map'
 
 
-def mk_web_app(funcs: Iterable[Callable], *, api_url: str = None, **kwargs):
+def mk_web_app(
+    funcs: Iterable[Callable], *, api: HttpClient = None, api_url: str = None, **kwargs
+):
     """Generates a front application which will consume a web service exposing a bunch
     of functions.
 
     :param funcs: A list of functions.
-    :param api_url: The base url of the API.
+    :param api: The HttpClient object to consume the API.
+    :param api_url: The base url of the API to create a HttpClient object in case the 
+    `api` parameter is not provided. Ignored otherwise
     :param kwargs: Any extra keyword argument used to make the front application.
     """
 
@@ -50,13 +54,6 @@ def mk_web_app(funcs: Iterable[Callable], *, api_url: str = None, **kwargs):
 
         crude_config = {k: v for k, v in build_crude_config()}
         if crude_config:
-            if not hasattr(api, 'get_light_mall'):
-                raise RuntimeError(
-                    'Some parameters have been crudified but there is \
-no way to get the list of valid keys for them. Make sure to expose the \
-"get_light_mall" endpoint through the API'
-                )
-            light_mall = api.get_light_mall()
             config = kwargs.get('config', {})
             for func_name, param_to_mall_map in crude_config.items():
                 for param, store in param_to_mall_map.items():
@@ -65,13 +62,18 @@ no way to get the list of valid keys for them. Make sure to expose the \
                     )
                     param_config = glom.glom(config, path, default={})
                     if ELEMENT_KEY not in param_config:
+                        if not hasattr(api, 'get_store_keys'):
+                            raise RuntimeError(
+                                'Some parameters have been crudified but there is no way to get the list of valid keys for them. Make sure to expose the "get_store_keys" endpoint through the API'
+                            )
                         param_config[ELEMENT_KEY] = SelectBox
-                        param_config['options'] = list(light_mall.get(store, {}))
+                        param_config['options'] = api.get_store_keys(store)
                         glom.assign(config, path, param_config, missing=dict)
             kwargs['config'] = config
 
-    openapi_spec_url = urljoin(api_url, 'openapi')
-    api = HttpClient(url=openapi_spec_url)
+    if not api:
+        openapi_spec_url = urljoin(api_url, 'openapi')
+        api = HttpClient(url=openapi_spec_url)
     func_names = [name_of_obj(func) for func in funcs]
     ws_funcs = [flatten_api_meth(getattr(api, name)) for name in func_names]
     handle_crudified_params()
@@ -93,26 +95,24 @@ def mk_api(funcs: Iterable[Callable], mall: Optional[Mall] = None, **kwargs):
 
     if mall is not None:
 
-        def get_light_mall():
-            def remove_values(node):
-                return {
-                    k: (remove_values(v) if isinstance(v, Mapping) else None)
-                    for k, v in node.items()
-                }
+        def get_store_keys(store_name_: str = None):
+            if store_name_ is None:
+                return {k: get_store_keys(k) for k in mall}
+            if store_name_ not in mall:
+                raise ValueError(f'No store named "{store_name_}"')
+            return list(mall[store_name_])
 
-            return remove_values(mall)
+        funcs = funcs + [get_store_keys]
 
-        funcs = funcs + [get_light_mall]
-
-    ws_config = dict(
+    dflt_config = dict(
         protocol='http',
         host='localhost',
         port='3030',
         enable_cors=True,
         publish_openapi=True,
         publish_swagger=True,
-        **kwargs,
     )
+    ws_config = dict(dflt_config, **kwargs)
 
     if 'openapi' not in ws_config:
         protocol = ws_config['protocol']
@@ -123,5 +123,5 @@ def mk_api(funcs: Iterable[Callable], mall: Optional[Mall] = None, **kwargs):
     return mk_webservice(funcs, **ws_config)
 
 
-def run_api(app):
-    run_webservice(app)
+def run_api(app, **kwargs):
+    run_webservice(app, **kwargs)
